#include "particlesystem.h"
#include "particlemanager.h"
#include "particleobject.h"

void CMMParticleSystem::CleanUp()
{
	for( int i = 0; i < particlenum; ++i )
	{
		SAFE_RELEASE( particles[i] );
	};
	SAFE_DELETE_ARRAY( particles );
	particlenum = 0;
	particlesize = 0.0f;
	initparticle = NULL;
	updateparticle = NULL;
	isparticlealive = NULL;
	SAFE_RELEASE( texture );
	SAFE_RELEASE( vertexbuffer );
	SAFE_DELETE_ARRAY( buffer );
	animated = false;
};

void CMMParticleSystem::Reset( LPDIRECT3DDEVICE9 DEVICE,
							   int PARTICLENUM, float PARTICLESIZE,
							   LPInitParticle INITPARTICLE,
							   LPUpdateParticle UPDATEPARTICLE,
							   LPIsParticleAlive ISPARTICLEALIVE,
							   CMMTexture* TEXTURE, bool ANIMATED )
{
	CleanUp();
	particlenum = PARTICLENUM;
	particlesize = PARTICLESIZE;
	initparticle = INITPARTICLE;
	updateparticle = UPDATEPARTICLE;
	isparticlealive = ISPARTICLEALIVE;
	particles = new CMMParticle*[particlenum];
	buffer = new PARTICLEVERTEX[particlenum];
	DEVICE->CreateVertexBuffer( particlenum * sizeof( PARTICLEVERTEX ),
		0, D3DFVF_PARTICLEVERTEX, D3DPOOL_DEFAULT, &vertexbuffer, NULL );
	for( int i = 0; i < particlenum; ++i )
	{
		particles[i] = Particles::ParticleManager->CreateParticleObject();
		initparticle( this, i, true );
		buffer[i].position = particles[i]->GetPosition();
		buffer[i].color = particles[i]->GetColor();
	};
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::Update( int UPDATES )
{
	for( int i = 0; i < particlenum; ++i )
	{
		updateparticle( this, i );
		if( !isparticlealive( this, i ) ) initparticle( this, i, false );
		buffer[i].position = particles[i]->GetPosition();
		buffer[i].color = particles[i]->GetColor();
	};
};

CMMParticle* CMMParticleSystem::GetParticle( int PARTICLEINDEX )
{
	if( PARTICLEINDEX >=0 && PARTICLEINDEX < particlenum) return particles[PARTICLEINDEX];
	else return NULL;
};

void CMMParticleSystem::SetTexture( CMMTexture* TEXTURE, bool ANIMATED )
{
	SAFE_RELEASE( texture );
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::Draw( LPDIRECT3DDEVICE9 DEVICE )
{
	float zero = 0.0f;
	float one = 1.0f;
	if( !texture ) return;
	DEVICE->SetFVF( D3DFVF_PARTICLEVERTEX );
	DEVICE->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
	DEVICE->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_FLAT );
	DEVICE->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE );
	DEVICE->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
	DEVICE->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	DEVICE->SetRenderState( D3DRS_LIGHTING, false );
	DEVICE->SetRenderState( D3DRS_POINTSPRITEENABLE, true );
	DEVICE->SetRenderState( D3DRS_POINTSCALEENABLE, true );
	DEVICE->SetRenderState(D3DRS_POINTSCALE_A, *((DWORD*)&zero));
	DEVICE->SetRenderState(D3DRS_POINTSCALE_B, *((DWORD*)&zero));
	DEVICE->SetRenderState(D3DRS_POINTSCALE_C, *((DWORD*)&one));
	DEVICE->SetRenderState( D3DRS_POINTSIZE, *((DWORD*)&particlesize));
	DEVICE->SetRenderState( D3DRS_ALPHABLENDENABLE, true );
	DEVICE->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	DEVICE->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
	DEVICE->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
	DEVICE->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
	DEVICE->SetTexture( 0, texture->GetTexture() );
	void* pVertices;
	vertexbuffer->Lock( 0, sizeof( buffer ), ( void** )&pVertices, 0 );
	memcpy( pVertices, buffer, sizeof( buffer ) );
	vertexbuffer->Unlock();
	DEVICE->SetStreamSource( 0, vertexbuffer, 0, sizeof( PARTICLEVERTEX ) );
	DEVICE->SetFVF( D3DFVF_PARTICLEVERTEX );
	DEVICE->DrawPrimitive( D3DPT_POINTLIST, 0, particlenum );
};